
#include "p_gpio.h"

#include <zephyr/device.h>
#include <zephyr/drivers/gpio.h>

#include <zephyr/kernel.h>

#include <zephyr/logging/log.h>

LOG_MODULE_REGISTER(p_gpio);

static const struct device* gpio_0;
static const struct device* gpio_1;

int p_gpio_set_pin_internal(const struct device* gpio, uint8_t pin, bool value);
static int p_gpio_pin_callback_init_internal(const struct device* gpio, uint8_t pin, uint32_t pin_flags, gpio_callback_handler_t callback, struct gpio_callback* callback_data, uint32_t callback_flags);

int p_gpio_init() {
	gpio_0 = device_get_binding("GPIO_0");
	gpio_1 = device_get_binding("GPIO_1");
	
	int res = p_gpio_init_pins();
	
	if(!gpio_0 || !gpio_1 || res) {
		LOG_ERR("Error: Failed to init GPIO pins");
	}

	return res;
}

int p_gpio_init_pins() {
	int res;
	//Test Setup Pins
	res = p_gpio_pin_init(gpio_0, 31, GPIO_OUTPUT | GPIO_PULL_UP);
	gpio_port_clear_bits(gpio_0, BIT(31));

	res |= p_gpio_pin_init(gpio_0, 30, GPIO_OUTPUT | GPIO_PULL_UP);
	gpio_port_clear_bits(gpio_0, BIT(30));

	res |= p_gpio_pin_init(gpio_0, 29, GPIO_OUTPUT | GPIO_PULL_UP);
	gpio_port_clear_bits(gpio_0, BIT(29));

	return res;
}

int p_gpio_get_pin(uint8_t gpio, uint8_t pin) {
	if(!gpio) {
		return gpio_pin_get(gpio_0, pin);
	} else {
		return gpio_pin_get(gpio_1, pin);
	}
	
}

int p_gpio_set_pin(uint8_t gpio, uint8_t pin, bool value) {
	if(!gpio) {
		return p_gpio_set_pin_internal(gpio_0, pin, value);
	} else {
		return p_gpio_set_pin_internal(gpio_1, pin, value);
	}
}

int p_gpio_set_pin_internal(const struct device* gpio, uint8_t pin, bool value) {
	if(value) {
		return gpio_port_set_bits(gpio, BIT(pin));
	} else {
		return gpio_port_clear_bits(gpio, BIT(pin));
	}
	
}

int p_gpio_pin_init(const struct device* gpio, uint8_t pin, uint32_t flags) {
	int err = gpio_pin_configure(gpio, pin, flags);
	if(err) {
		LOG_ERR("Failed to init pin %d", pin);
	} else {
		LOG_INF("Pin %d inited", pin);
	}
	return err;
}

int p_gpio_pin_callback_init(uint8_t gpio, uint8_t pin, uint32_t pin_flags, gpio_callback_handler_t callback, struct gpio_callback* callback_data, uint32_t callback_flags) {	
	if(!gpio) {
		return p_gpio_pin_callback_init_internal(gpio_0, pin, pin_flags, callback, callback_data, callback_flags);
	} else {
		return p_gpio_pin_callback_init_internal(gpio_1, pin, pin_flags, callback, callback_data, callback_flags);
	}
}

int p_gpio_pin_callback_init_internal(const struct device* gpio, uint8_t pin, uint32_t pin_flags, gpio_callback_handler_t callback, struct gpio_callback* callback_data, uint32_t callback_flags) {
	int ret;
	ret = p_gpio_pin_init(gpio, pin, pin_flags);
	if(ret != 0) {
		LOG_ERR("Error %d: failed to configure pin %d\n", ret, pin);
		return ret;
	}

	ret = gpio_pin_interrupt_configure(gpio, pin, callback_flags);
	if(ret != 0) {
		LOG_ERR("Error %d: failed to configure interrupt on pin %d\n", ret, pin);
		return ret;
	}

	gpio_init_callback(callback_data, callback, BIT(pin));
	gpio_add_callback(gpio, callback_data);
	LOG_INF("Set up interrupt on pin %d\n", pin);
	return ret;
}